local addonName = ...
local LockboxMailer = CreateFrame("Frame")

function LockboxMailer:SaveRecipient()
    if self.isClassic then
        local playerName = SendMailNameEditBox:GetText()
        local realmName = GetNormalizedRealmName()
        if playerName ~= "" then
            LockboxMailerDB.realms[realmName] = playerName
        end
    else
        local playerName, realm = strsplit("-", SendMailNameEditBox:GetText())
        local connectedRealms = GetAutoCompleteRealms()
        local normalizedRealmName = GetNormalizedRealmName()
        local faction = UnitFactionGroup("player")
        realm = realm or normalizedRealmName
        if playerName ~= "" then
            if connectedRealms and LockboxMailerDB.options.saveConnectedRealms then
                for i=1, #connectedRealms do
                    local connectedRealm = connectedRealms[i]
                    LockboxMailerDB.realms[connectedRealm..":"..faction] = playerName.."-"..realm
                end
            else
                LockboxMailerDB.realms[normalizedRealmName..":"..faction] = playerName.."-"..realm
            end
        end
    end
end

function LockboxMailer:SetRecipient()
    local realmName = GetNormalizedRealmName()
    if self.isClassic then
        if realmName then
            local target = LockboxMailerDB.realms[realmName]
            if target then
                return target
            end
        end
    else
        local connectedRealms = GetAutoCompleteRealms()
        local unitFaction = UnitFactionGroup("player")
        if realmName then
            if connectedRealms then
                for i=1, #connectedRealms do
                    local connectedRealm = connectedRealms[i]
                    local target = LockboxMailerDB.realms[connectedRealm..":"..unitFaction]
                    if target then
                        return target
                    end
                end
            end
            local target = LockboxMailerDB.realms[realmName..":"..unitFaction]
            if target then
                return target
            end
        end
    end
end

do
    local LockBoxButton = CreateFrame("Button", "LockBoxMailButton", SendMailFrame, "ActionButtonTemplate")
    LockBoxButton:ClearAllPoints()
    LockBoxButton:SetPoint("RIGHT", SendMailFrame, "RIGHT", -56, -126)
    LockBoxButton:SetSize(24,24)
    LockBoxButton:RegisterForClicks("AnyUp")
    LockBoxButton:SetScript("OnClick", function(_,btn)
        if btn == "RightButton" then
            LockboxMailer:SaveRecipient()
        else
            LockboxMailer:ProcessMailing(true)
        end
    end)
    LockBoxButton.icon:SetTexture(134344)
    LockBoxButton.NormalTexture:ClearAllPoints()
    LockBoxButton.tooltip = "|TInterface\\HELPFRAME\\NewPlayerExperienceParts:24:24:0:0:1024:512:981:1013:66:98|t一键发送宝箱\n|TInterface\\HELPFRAME\\NewPlayerExperienceParts:24:24:0:0:1024:512:981:1013:132:164|t保存收件人"
    LockBoxButton:SetScript("OnEnter", function(self)
        if self.tooltip then
            GameTooltip:SetOwner(self, "ANCHOR_NONE");
            GameTooltip:SetPoint("LEFT", self, "RIGHT", 12, 0)
            GameTooltip:SetText(self.tooltip, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, 1, true);
            GameTooltip:Show();
        end
    end)
    LockBoxButton:SetScript("OnLeave", GameTooltip_Hide)
end

local tooltip = CreateFrame("GameTooltip","LockboxMailerTooltip", UIParent, "GameTooltipTemplate");
tooltip:SetOwner(LockboxMailer, "ANCHOR_NONE");

function LockboxMailer:FindLockboxes()
    wipe(self.LockboxTable)
    for bag = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots(bag) do

            local lootable, itemLink = select(6,GetContainerItemInfo(bag, slot))
            if itemLink then
                local bindType = select(14, GetItemInfo(itemLink))

                if bindType ~= 1 and lootable then
                    tooltip:SetBagItem(bag, slot)
                    if tooltip:IsShown() then
                        for i = 1, tooltip:NumLines() do
                            local line = _G["LockboxMailerTooltipTextLeft"..i]:GetText()
                            if (line == LOCKED) or (LockboxMailerDB.options.sendUnlocked and (line == ITEM_OPENABLE)) then
                                tinsert(self.LockboxTable, { bag, slot } )
                            end
                        end
                    end
                end
            end

        end
    end
end


function LockboxMailer:ProcessMailing(click)
    local toPlayerName = SendMailNameEditBox:GetText()
    local playerName = UnitName("player")
    local realmName = GetNormalizedRealmName()
    local recipient = self:SetRecipient()
    recipient = toPlayerName ~= "" and toPlayerName or recipient

    if not recipient or (recipient and recipient == (realmName and playerName.."-"..realmName) or recipient == playerName) then
        if not click and LockboxMailerDB.options.autoSendOnMailOpen then
            return
        end
    end

    if LockboxMailerDB.options.autoClearAttachment then
        ClearSendMail()
        SendMailNameEditBox:SetText(toPlayerName)
    end
    local attachmentIndex = 0
    for i = 1, ATTACHMENTS_MAX_SEND do
        if GetSendMailItem(i) then
            attachmentIndex = attachmentIndex + 1
        end
    end

    if attachmentIndex < ATTACHMENTS_MAX_SEND then
        self:FindLockboxes()
    end

    local quantity = #self.LockboxTable
    if next(self.LockboxTable) then
        attachmentIndex = (attachmentIndex == 0 and 1 or attachmentIndex + 1)
        for i = 1, quantity do
            local item = tremove(self.LockboxTable)
            if attachmentIndex == ATTACHMENTS_MAX_SEND then
                UIErrorsFrame:AddMessage(ERR_MAIL_INVALID_ATTACHMENT_SLOT, 1.0, 0.1, 0.1, 1.0)
                return
            end
            ClearCursor()
            PickupContainerItem(item[1], item[2])
            ClickSendMailItemButton()
            attachmentIndex = attachmentIndex + 1
        end
        SendMailSubjectEditBox:SetText("Lockbox Mailer: ("..quantity..")")

        local playerName = UnitName("player")
        local realmName = GetNormalizedRealmName()
        local recipient = self:SetRecipient()
        recipient = toPlayerName ~= "" and toPlayerName or recipient

        if recipient then
            if recipient == (realmName and playerName.."-"..realmName) or recipient == playerName then
                UIErrorsFrame:AddMessage(ERR_MAIL_TO_SELF.."\n Save a new recipient first.", 1.0, 0.1, 0.1, 1.0)
                return
            end

            SendMailNameEditBox:SetText(recipient)
            if click and LockboxMailerDB.options.autoSendAfterButtonPush or (not click and LockboxMailerDB.options.autoSendOnMailOpen) then
                if recipient == playerName then
                    UIErrorsFrame:AddMessage(ERR_MAIL_TO_SELF, 1.0, 0.1, 0.1, 1.0)
                    return
                end
                SendMail(recipient, "Lockbox Mailer: ("..quantity..")", "")
            end
        end
    end
end

function LockboxMailer:OnEvent(event, ...)
    if event == "ADDON_LOADED" and addonName == ... then
        self.LockboxTable = {}
        LockboxMailerDB = LockboxMailerDB or {}
        LockboxMailerDB.realms = LockboxMailerDB.realms or {}
        LockboxMailerDB.options = LockboxMailerDB.options or {}
    elseif event == "MAIL_SHOW" then
        if LockboxMailerDB.options.autoSendOnMailOpen then
            C_Timer.After(1, function () LockboxMailer:ProcessMailing() end)
        end
    end
end

function LockboxMailer:OnLoad()
    self:SetScript("OnEvent", self.OnEvent)
    self:RegisterEvent("MAIL_SHOW")
    self:RegisterEvent("ADDON_LOADED")
    self.isClassic = (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC)
end
LockboxMailer:OnLoad()

function LockboxMailer:Help(msg)
    local fName = "|cffEEE4AELockbox Mailer:|r "
    local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
    if not cmd or cmd == "" or cmd == "help" then
        print(fName.."   |cff58C6FA/lmail    /lockboxmailer|r")
        print("  |cff58C6FA/lmail all    -|r  |cffEEE4AEAlso send Lockboxes that are no longer locked. (toggle)|r")
        if not self.isClassic then
            print("  |cff58C6FA/lmail connected    -|r  |cffEEE4AEWhen saving recipient, save for all connected realms (toggle)|r")
        end
        print("  |cff58C6FA/lmail autobutton    -|r  |cffEEE4AESend Mail automatically after pressing lockbox button (toggle)|r")
        print("  |cff58C6FA/lmail automail      -|r  |cffEEE4AESend Lockboxes automatically when opening mailbox (toggle)|r")
        print("  |cff58C6FA/lmail autoclear      -|r  |cffEEE4AEClear Attachments before adding lockboxes|r")
        print("  |cff58C6FA/lmail status        -|r  |cffEEE4AEShow current state of options|r")
        print("  |cff58C6FA/lmail reset        -|r  |cffEEE4AEReset everything to default and delete recipients.")
    elseif cmd == "all" then
        if LockboxMailerDB.options.sendUnlocked then
            LockboxMailerDB.options.sendUnlocked = false
            print(fName.."|cffB6B6B6Only sending locked lockboxes.")
        else
            LockboxMailerDB.options.sendUnlocked = true
            print(fName.."|cff37DB33Sending unlocked lockboxes too.")
        end
    elseif cmd == "connected" then
        if self.isClassic then
            if LockboxMailerDB.options.saveConnectedRealms then
                LockboxMailerDB.options.saveConnectedRealms = false
                print(fName.."|cffB6B6B6Save recipient for all connected realms DISABLED.")
            else
                LockboxMailerDB.options.saveConnectedRealms = true
                print(fName.."|cff37DB33Save recipient for all connected realms ENABLED.")
            end
        end
    elseif cmd == "autobutton" then
        if LockboxMailerDB.options.autoSendAfterButtonPush then
            LockboxMailerDB.options.autoSendAfterButtonPush = false
            print(fName.."|cffB6B6B6Send Mail after pushing Lockbox Mailer button DISABLED.")
        else
            LockboxMailerDB.options.autoSendAfterButtonPush = true
            print(fName.."|cff37DB33Send Mail after pushing Lockbox Mailer button ENABLED.")
        end
    elseif cmd == "automail" then
        if LockboxMailerDB.options.autoSendOnMailOpen then
            LockboxMailerDB.options.autoSendOnMailOpen = false
            print(fName.."|cffB6B6B6Automatically send Lockboxes when opening Mailbox DISABLED.")
        else
            LockboxMailerDB.options.autoSendOnMailOpen = true
            print(fName.."|cff37DB33Automatically send Lockboxes when opening Mailbox ENABLED.")
        end
    elseif cmd == "autoclear" then
        if LockboxMailerDB.options.autoClearAttachment then
            LockboxMailerDB.options.autoClearAttachment = false
            print(fName.."|cffB6B6B6Automatically clear attachments DISABLED.")
        else
            LockboxMailerDB.options.autoClearAttachment = true
            print(fName.."|cff37DB33Automatically clear attachments ENABLED.")
        end
    elseif cmd == "status" then
        if LockboxMailerDB.options.sendUnlocked then
            print(fName.."|cff37DB33Sending unlocked lockboxes too.")
        else
            print(fName.."|cffB6B6B6Only sending locked lockboxes.")
        end

        if self.isClassic then
            if LockboxMailerDB.options.saveConnectedRealms then
                print(fName.."|cff37DB33Save recipient for all connected realms ENABLED.")
            else
                print(fName.."|cffB6B6B6Save recipient for all connected realms DISABLED.")
            end
        end

        if LockboxMailerDB.options.autoSendOnMailOpen then
            print(fName.."|cff37DB33Automatically send Lockboxes when opening Mailbox ENABLED.")
        else
            print(fName.."|cffB6B6B6Automatically send Lockboxes when opening Mailbox DISABLED.")
        end

        if LockboxMailerDB.options.autoSendAfterButtonPush then
            print(fName.."|cff37DB33Send Mail after pushing Lockbox Mailer button ENABLED.")
        else
            print(fName.."|cffB6B6B6Send Mail after pushing Lockbox Mailer button DISABLED.")
        end

        if LockboxMailerDB.options.autoClearAttachment then
            print(fName.."|cff37DB33Automatically clear attachments ENABLED.")
        else
            print(fName.."|cffB6B6B6Automatically clear attachments DISABLED.")
        end
    elseif cmd == "reset" then
        wipe(LockboxMailerDB)
        LockboxMailerDB.realms = {}
        LockboxMailerDB.options = {}
    end
end

SLASH_LOCKBOXMAILER1, SLASH_LOCKBOXMAILER2  = "/lmail", "/lockboxmailer"
SlashCmdList["LOCKBOXMAILER"] = function(...)
    LockboxMailer:Help(...)
end